@@ -3,20 +3,20 @@ use strict;
 use warnings;
 use OptArgs;
 use lib 'lib';
-our $VERSION = '0.1.16';
+our $VERSION = '0.1.17_1';
 
 $OptArgs::COLOUR = 1;
 
 arg class => (
     isa      => 'Str',
     required => 1,
-    comment  => 'OptArgs-based module to map',
+    comment  => 'OptArgs-based module to load',
 );
 
 arg name => (
     isa     => 'Str',
     comment => 'Name of the command',
-    default => sub { ( my $x = shift->{class} ) =~ s/.*://; $x; }
+    default => sub { '<command>' }
 );
 
 opt indent => (
@@ -49,7 +49,7 @@ sub run {
 
     binmode( STDOUT, ':encoding(utf8)' );
 
-    foreach my $cmd ( OptArgs::_cmdlist( $opts->{class} ) ) {
+    foreach my $cmd ( OptArgs::_cmdlist() ) {
         my $length = do { my @tmp = split( /::/, $cmd ) }
           - $initial;
         my $space = $indent x $length;
@@ -84,7 +84,7 @@ App::optargs - implementation of the optargs(1) command
 
 =head1 VERSION
 
-0.1.16 development release.
+0.1.17_1 development release.
 
 =head1 SYNOPSIS
 
