@@ -9,7 +9,7 @@ use Exporter::Tidy
 use Getopt::Long qw/GetOptionsFromArray/;
 use List::Util qw/max/;
 
-our $VERSION = '0.1.16';
+our $VERSION = '0.1.17_1';
 our $COLOUR  = 0;
 our $ABBREV  = 0;
 our $SORT    = 0;
@@ -22,33 +22,9 @@ my %desc;           # sub-command descriptions
 my %dispatching;    # track optargs() calls from dispatch classes
 my %hidden;         # subcmd hiding by default
 
-# internal method for App::optargs
+# internal function for App::optargs
 sub _cmdlist {
-    my $package = shift || croak '_cmdlist($package)';
-    $package =~ s/-/_/g;
-    my @list = ($package);
-
-    if ( exists $args{$package} ) {
-        my @subcmd =
-          map { exists $_->{subcommands} ? $_->{subcommands} : () }
-          @{ $args{$package} };
-
-        push( @subcmd,
-            map { exists $_->{fallback} ? [ uc $_->{fallback}->{name} ] : () }
-              @{ $args{$package} } );
-
-        if ($SORT) {
-            foreach my $subcmd ( sort map { @$_ } @subcmd ) {
-                push( @list, _cmdlist( $package . '::' . $subcmd ) );
-            }
-        }
-        else {
-            foreach my $subcmd ( map { @$_ } @subcmd ) {
-                push( @list, _cmdlist( $package . '::' . $subcmd ) );
-            }
-        }
-    }
-    return @list;
+    return sort grep { $_ ne 'App::optargs' } keys %seen;
 }
 
 # ------------------------------------------------------------------------
@@ -352,6 +328,11 @@ sub _usage {
         next if $opt->{hidden} and !$ishelp;
 
         ( my $name = $opt->{name} ) =~ s/_/-/g;
+
+        if ( $opt->{isa} eq 'Bool' and $opt->{default} ) {
+            $name = 'no-' . $name;
+        }
+
         $name .= ',' if $opt->{alias};
         push(
             @uopts,
@@ -436,9 +417,10 @@ sub usage {
 # Option/Argument processing
 # ------------------------------------------------------------------------
 sub _optargs {
-    my $caller  = shift;
-    my $source  = \@_;
-    my $package = $caller;
+    my $caller      = shift;
+    my $source      = \@_;
+    my $source_hash = {};
+    my $package     = $caller;
 
     if ( !@_ and @ARGV ) {
         my $CODESET =
@@ -451,6 +433,10 @@ sub _optargs {
 
         $source = \@ARGV;
     }
+    else {
+        $source_hash = { map { %$_ } grep { ref $_ eq 'HASH' } @$source };
+        $source = [ grep { ref $_ ne 'HASH' } @$source ];
+    }
 
     map { Carp::croak('_optargs argument undefined!') if !defined $_ } @$source;
 
@@ -471,7 +457,12 @@ sub _optargs {
         my $result;
 
         if ( $try->{type} eq 'opt' ) {
-            GetOptionsFromArray( $source, $try->{ISA} => \$result );
+            if ( exists $source_hash->{ $try->{name} } ) {
+                $result = delete $source_hash->{ $try->{name} };
+            }
+            else {
+                GetOptionsFromArray( $source, $try->{ISA} => \$result );
+            }
         }
         elsif ( $try->{type} eq 'arg' ) {
             if (@$source) {
@@ -518,6 +509,9 @@ sub _optargs {
 
                 # TODO: type check using Param::Utils?
             }
+            elsif ( exists $source_hash->{ $try->{name} } ) {
+                $result = delete $source_hash->{ $try->{name} };
+            }
             elsif ( $try->{required} and !$ishelp ) {
                 $missing_required++;
                 next;
@@ -582,8 +576,11 @@ sub _optargs {
         die _usage($package);
     }
     elsif (@$source) {
+        die _usage( $package, "Unexpected options or arguments: @$source" );
+    }
+    elsif ( my @unexpected = keys %$source_hash ) {
         die _usage( $package,
-            qq{Unexpected option or argument "} . ( shift @$source ) . '"' );
+            "Unexpected HASH options or arguments: @unexpected" );
     }
 
     # Re-calculate the default if it was a subref
@@ -613,27 +610,7 @@ sub class_optargs {
 
     die $@ unless eval "require $caller;";
 
-    my ( $class, $optargs );
-
-    if (@_) {
-        my @args;
-
-        foreach my $element (@_) {
-            if ( ref $element eq 'HASH' ) {
-                push( @args,
-                    '--' . $_, defined $element->{$_} ? $element->{$_} : () )
-                  for keys %$element;
-            }
-            else {
-                push( @args, $element );
-            }
-        }
-
-        ( $class, $optargs ) = _optargs( $caller, @args );
-    }
-    else {
-        ( $class, $optargs ) = _optargs($caller);
-    }
+    my ( $class, $optargs ) = _optargs( $caller, @_ );
 
     croak $@ unless eval "require $class;1;";
     return ( $class, $optargs );
@@ -648,14 +625,7 @@ sub dispatch {
 
     my ( $package, $optargs ) = class_optargs( $class, @_ );
 
-    croak $@ unless eval "require $package";
-
     my $sub = $package->can($method);
-    if ( !$sub ) {
-        croak $@ unless eval "require $package;";
-        $sub = $package->can($method);
-    }
-
     die "Can't find method $method via package $package" unless $sub;
 
     $dispatching{$class}++;
@@ -681,7 +651,7 @@ OptArgs - integrated argument and option processing
 
 =head1 VERSION
 
-0.1.16 (2014-10-01)
+0.1.17_1 (2015-10-22)
 
 =head1 SYNOPSIS
 
@@ -1104,6 +1074,12 @@ containg all option/argument values after parsing the source has
 finished.  The value to be set must be returned, and any changes to the
 hashref are ignored.
 
+For "Bool" options setting "default" to a true has a special effect:
+the the usage message formats it as "--no-option" instead of
+"--option". If you do use a true default value for Bool options you
+probably want to reverse the normal meaning of your "comment" value as
+well.
+
 =item alias
 
 A single character alias.
